<?php

use JsonSchema\Validator;
use Zen\Payment\Util;

/**
 * Class ControllerExtensionPaymentZen
 */
class ControllerExtensionPaymentZen extends Controller
{
	private $error = [];

	/**
	 * view admin panel
	 */
	public function index()
	{
		$this->language->load('extension/payment/zen');
		$this->document->setTitle($this->language->get('heading_title'));
		$this->document->addScript('view/javascript/select2/js/select2.min.js');
		$this->document->addStyle('view/javascript/select2/css/select2.css');
		$this->load->model('setting/setting');

		$request = $this->request->post;

		if(($this->request->server['REQUEST_METHOD'] == 'POST')
			&& $this->validate($request)
		) {

			$settings = array_merge($this->model_setting_setting->getSetting('zen'), $request);
			$this->model_setting_setting->editSetting('zen', $settings);
			$this->session->data['success'] = $this->language->get('text_success');
			$this->response->redirect($this->url->link('extension/extension', 'token=' . $this->session->data['token'], 'SSL'));

			return;
		}

		include_once(DIR_SYSTEM . 'library/payment-core/vendor/autoload.php');

		$data = [

			'button_save'   => $this->language->get('button_save'),
			'button_cancel' => $this->language->get('button_cancel'),

			'text_edit' => $this->language->get('text_edit'),

			'text_enabled'  => $this->language->get('text_enabled'),
			'text_disabled' => $this->language->get('text_disabled'),

			'text_header_tip' => $this->language->get('text_header_tip'),

			'zen_status_label' => $this->language->get('zen_status_label'),
			'zen_status'       => isset($this->request->post['zen_status'])
				? $this->request->post['zen_status']
				: $this->config->get('zen_status'),

			'zen_sandbox_label'        => $this->language->get('zen_sandbox_label'),
			'zen_sandbox_enabled_hint' => $this->language->get('zen_sandbox_enabled_hint'),
			'zen_sandbox'              => isset($this->request->post['zen_sandbox'])
				? $this->request->post['zen_sandbox']
				: $this->config->get('zen_sandbox'),

			'zen_terminal_uuid_label' => $this->language->get('zen_terminal_uuid_label'),
			'zen_terminal_uuid'       => isset($this->request->post['zen_terminal_uuid'])
				? $this->request->post['zen_terminal_uuid']
				: $this->config->get('zen_terminal_uuid'),

			'zen_paywall_secret_label' => $this->language->get('zen_paywall_secret_label'),
			'zen_paywall_secret'       => isset($this->request->post['zen_paywall_secret'])
				? $this->request->post['zen_paywall_secret']
				: $this->config->get('zen_paywall_secret'),

			'zen_ipn_secret_label' => $this->language->get('zen_ipn_secret_label'),
			'zen_ipn_secret'       => isset($this->request->post['zen_ipn_secret'])
				? $this->request->post['zen_ipn_secret']
				: $this->config->get('zen_ipn_secret'),

			'zen_sort_order_label' => $this->language->get('zen_sort_order_label'),
			'zen_sort_order'       => isset($this->request->post['zen_sort_order'])
				? $this->request->post['zen_sort_order']
				: $this->config->get('zen_sort_order'),

			'zen_payment_title_default' => $this->language->get('zen_payment_title_default'),
			'zen_payment_title_label'   => $this->language->get('zen_payment_title_label'),
			'zen_payment_title'         => isset($this->request->post['zen_payment_title'])
				? $this->request->post['zen_payment_title']
				: $this->config->get('zen_payment_title'),

			'zen_currency_label' => $this->language->get('zen_currency_label'),
			'zen_all_currencies' => Util::getCurrencies(),
			'zen_currencies'     => isset($this->request->post['zen_currencies'])
				? $this->request->post['zen_currencies']
				: $this->config->get('zen_currencies')
					? $this->config->get('zen_currencies')
					: [],

			'text_footer_notification' => $this->language->get('text_footer_notification'),
			'text_notification_url'    => HTTPS_CATALOG . 'index.php?route=extension/payment/zen/notification',

			'confirm_url' => $this->url->link('extension/payment/zen', 'token=' . $this->session->data['token'], true),
			'cancel_url'  => $this->url->link('extension/extension', 'token=' . $this->session->data['token'], true),

			'content_footer'      => $this->load->controller('common/footer'),
			'content_header'      => $this->load->controller('common/header'),
			'content_column_left' => $this->load->controller('common/column_left'),
			'heading_title'       => $this->language->get('text_heading_title'),

			'breadcrumbs' => [
				[
					'text'      => $this->language->get('text_home'),
					'href'      => $this->url->link('common/home', 'token=' . $this->session->data['token'], 'SSL'),
					'separator' => false,
				],

				[
					'text'      => $this->language->get('text_payment'),
					'href'      => $this->url->link('extension/extension', 'token=' . $this->session->data['token'], 'SSL'),
					'separator' => ' :: ',
				],

				[
					'text'      => $this->language->get('text_heading_title'),
					'href'      => $this->url->link('extension/payment/zen', 'token=' . $this->session->data['token'], 'SSL'),
					'separator' => ' :: ',
				],
			],

		];

		$this->response->setOutput($this->load->view('extension/payment/zen', $data));
	}

	/**
	 * @param array $request
	 *
	 * @return bool
	 */
	protected function validate($request)
	{
		if(!$this->user->hasPermission('modify', 'extension/payment/zen')) {
			$this->error['error_warning_permissions'] = $this->language->get('error_permission');

			return false;
		}

		include_once(DIR_SYSTEM . 'library/payment-core/vendor/autoload.php');

		$schema = [
			'type'       => 'object',
			'properties' => [

				'zen_status'         => [
					'type' => 'string',
					"enum" => [
						'0',
						'1',
					],
				],
				'zen_sandbox'        => [
					'type' => 'string',
					"enum" => [
						'0',
						'1',
					],
				],
				'zen_terminal_uuid'  => [
					'type' => 'string',
				],
				'zen_paywall_secret' => [
					'type' => 'string',
				],
				'zen_ipn_secret'     => [
					'type' => 'string',
				],
				'zen_sort_order'     => [
					'type'      => 'string',
					'minLength' => 1,
				],
				'zen_payment_title'  => [
					'type' => 'string',
				],
				'zen_currencies'     => [
					'type'        => 'array',
					'items'       => [
						'type' => 'string',
						'enum' => array_values(Util::getCurrencies()),
					],
					'minItems'    => 1,
					'uniqueItems' => true,
				],
			],

			'required'             => [
				'zen_status',
				'zen_sandbox',
				'zen_terminal_uuid',
				'zen_paywall_secret',
				'zen_ipn_secret',
				'zen_sort_order',
				'zen_payment_title',
				'zen_currencies',
			],
			'additionalProperties' => false,
		];

		$data = json_decode(json_encode($request));

		$validator = new Validator();
		$validator->validate($data, $schema);

		$this->error = [];

		foreach($validator->getErrors() as $key => $value) {

			if($value['constraint'] === 'additionalProp') {
				$this->error['error_additional_properties'] = $this->language->get('error_additional_properties');
			} else {

				$this->error['error_' . $value['property']] = $this->language->get('error_validation');
			}
		}

		return empty($this->error);
	}
}
